package com.taauav.admin.controller;


import com.taauav.admin.query.TauvDriverQuery;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.admin.entity.TauvDriver;
import com.taauav.admin.service.ITauvDriverService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Map;

/**
 * <p>
 * 河道管理表 前端控制器
 * </p>
 *
 * @author dyg
 * @since 2019-11-12
 */
@RestController
@RequestMapping("/driver")
public class TauvDriverController {
    @Autowired
    private Response response;
    @Autowired
    private ITauvDriverService iTauvDriverService;

    private static final String controllerName = "driver";

    /**
     * 获取河道列表
     *
     * @param query 查询条件
     * @return
     */
    @PostMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(@RequestBody(required = false) TauvDriverQuery query) {
        return iTauvDriverService.getPageList(query);
    }

    /**
     * 修改河道数据
     *
     * @param driver
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody @Valid TauvDriver driver, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String msg = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(msg);
        }
        return iTauvDriverService.edit(driver);
    }

    /**
     * 获取河道数据
     *
     * @param id
     * @return
     */
    @GetMapping("/getDriverInfo")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response getDriverInfo(Integer id) {
        TauvDriver info = iTauvDriverService.getInfoById(id);
        if (StringUtils.isEmpty(info)) {
            return response.failure("数据不存在");
        }
        return response.success(iTauvDriverService.getInfoDetail(info));
    }

    /**
     * 添加河道数据
     *
     * @param driver
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody @Valid TauvDriver driver, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String msg = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(msg);
        }
        return iTauvDriverService.add(driver);
    }

    /**
     * 删除设备
     *
     * @param id 设备ID
     * @return
     */
    @PostMapping("/drop")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response drop(@RequestBody Integer id) {
        return iTauvDriverService.drop(id);
    }

    /**
     * 查看河道详细数据
     *
     * @param map
     * @return
     */
    @GetMapping("/detail")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.VIEW_PERMISSION)
    public Response detail(@RequestParam Map<String, String> map) {
        if (StringUtils.isEmpty(map)) {
            return response.failure("数据不存在");
        }
        if (!map.containsKey("id")) {
            return response.failure("参数不存");
        }
        Integer id = Integer.valueOf(map.get("id"));
        TauvDriver info = iTauvDriverService.getInfoById(id);
        if (StringUtils.isEmpty(info)) {
            return response.failure("数据不存在");
        }
        return response.success(iTauvDriverService.getInfoDetail(info));
    }

    /**
     * 基础数据
     *
     * @return
     */
    @GetMapping("/baseData")
    public Response getBaseData() {
        return response.success(iTauvDriverService.getBaseData());
    }

    /**
     * 获取所有河道数据
     *
     * @return
     */
    @GetMapping("/driverList")
    public Response getDriverList(@RequestParam Map<String, Object> map) {
        return iTauvDriverService.getDriverList(map);
    }
}
