package com.taauav.admin.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.entity.LsAdmin;
import com.taauav.admin.entity.TauvEquipment;
import com.taauav.admin.mapper.LsAdminMapper;
import com.taauav.admin.query.TauvDriverQuery;
import com.taauav.admin.service.ISysCityService;
import com.taauav.admin.vo.TauvDriverListVo;
import com.taauav.common.bean.Response;
import com.taauav.admin.entity.SysAdmin;
import com.taauav.admin.entity.TauvDriver;
import com.taauav.admin.mapper.TauvDriverMapper;
import com.taauav.admin.service.ISysAdminService;
import com.taauav.admin.service.ITauvDriverService;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 河道管理表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2019-11-12
 */
@Service
public class TauvDriverServiceImpl extends BaseServiceImpl<TauvDriverMapper, TauvDriver> implements ITauvDriverService {
    @Value("${spring.pageSize}")
    private Integer pageSize;
    @Autowired
    private Response response;

    @Autowired
    private ISysAdminService iSysAdminService;
    @Autowired
    private TauvDriverMapper driverMapper;
    @Autowired
    private ISysCityService cityService;
    @Autowired
    private LsAdminMapper lsAdminMapper;

    /**
     * 获取河道列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getPageList(TauvDriverQuery query) {
        // 查询条件
        QueryWrapper<TauvDriver> queryWrapper = new QueryWrapper<>();
        // 河道名称
        if (!StringUtils.isEmpty(query.getName())) {
            queryWrapper.like("name", query.getName());
        }
        // 河道类型
        if (query.getType() != null && query.getType() > 0) {
            queryWrapper.eq("type", query.getType());
        }
        // 河道层级
        if (query.getLevel() != null && query.getLevel() > 0) {
            queryWrapper.eq("level", query.getLevel());
        }
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByDesc("id");
        IPage<TauvDriver> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<TauvDriver> data = driverMapper.selectPage(page, queryWrapper);
        List<TauvDriver> driverList = data.getRecords();
        List<TauvDriverListVo> driverListVoList = new ArrayList<>();
        if (!driverList.isEmpty()) {
            driverList.forEach(item -> {
                TauvDriverListVo driverListVo = new TauvDriverListVo();
                // 拷贝属性
                BeanUtil.copyProperties(item, driverListVo);
                // 区划名称
                String cityName = cityService.getCityName(item.getDriverArea());
                driverListVo.setDriverAreaName(cityName);
                // 河长名称
                if (item.getDriverManager() != null && item.getDriverManager() > 0) {
                    LsAdmin lsAdmin = lsAdminMapper.selectById(item.getDriverManager());
                    if (lsAdmin != null) {
                        driverListVo.setDriverManagerName(lsAdmin.getRealname());
                    }
                }
                driverListVoList.add(driverListVo);
            });
        }

        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", driverListVoList);
        return response.success(result);
    }

    /**
     * 修改河道数据
     *
     * @param driver
     * @return
     */
    @Override
    public Response edit(TauvDriver driver) {
        if (driver == null) {
            return response.failure("数据不存在");
        }
        TauvDriver info = getInfoById(driver.getId());
        if (StringUtils.isEmpty(info)) {
            return response.failure("数据不存在");
        }
        boolean result = editData(driver);
        if (!result) {
            return response.failure("编辑失败");
        }
        return response.success("操作成功");
    }

    /**
     * 添加河道数据
     *
     * @param driver
     * @return
     */
    @Override
    public Response add(TauvDriver driver) {
        String name = driver.getName();
        TauvDriver info = getInfoByName(name);
        if (!StringUtils.isEmpty(info)) {
            return response.failure("河道名称已经存在");
        }
        boolean result = this.addData(driver);
        if (!result) {
            return response.failure("添加失败");
        }
        // 设置河湖编码
        String driverSn = String.format("%05d", driver.getId());
        driver.setDriverSn(driverSn);
        int rs = driverMapper.updateById(driver);
        if (rs == 0) {
            return response.failure("河湖编码更新失败");
        }
        return response.success("添加成功");
    }

    /**
     * 根据ID删除河道
     *
     * @param id 河道ID
     * @return
     */
    @Override
    public Response drop(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("河道ID不存在");
        }
        TauvDriver entity = driverMapper.selectById(id);
        if (entity == null) {
            return response.failure("河道信息不存在");
        }
        boolean result = this.deleteById(id, entity);
        if (!result) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }

    /**
     * 根据编号获取数据
     *
     * @param id
     * @return
     */
    @Override
    public TauvDriver getInfoById(Integer id) {
        return getInfo(id);
    }

    /**
     * 根据编号获取详细数据
     *
     * @param info
     * @return
     */
    @Override
    public Map<String, Object> getInfoDetail(TauvDriver info) {
        Map<String, Object> map = new HashMap<>();
        TauvDriver driver = new TauvDriver();
        map.put("driver_sn", info.getDriverSn());
        map.put("name", info.getName());
        map.put("type", info.getType());
        map.put("level", info.getLevel());
        map.put("driver_manager", info.getDriverManager());
        map.put("driver_area", info.getDriverArea());
        map.put("area_list", cityService.getCityListByPid(0));
        map.put("length", info.getLength());
        map.put("start_point", info.getStartPoint());
        map.put("end_point", info.getEndPoint());
        map.put("type_list", driver.getTypeList());
        map.put("format_create_time", DateUtil.format(info.getCreateTime(), "yyyy-MM-dd HH:mm:ss"));
        map.put("note", info.getNote());
        SysAdmin adminInfo = null;
        if (info.getCreateUser() > 0) {
            adminInfo = iSysAdminService.findById(info.getCreateUser());
        }
        map.put("format_create_user", StringUtils.isEmpty(adminInfo) ? "" : adminInfo.getRealname());
        return map;
    }

    /**
     * 根据河道名称获取数据
     *
     * @param name
     * @return
     */
    @Override
    public TauvDriver getInfoByName(String name) {
        QueryWrapper<TauvDriver> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("mark", 1);
        queryWrapper.eq("name", name);
        TauvDriver info = driverMapper.selectOne(queryWrapper);
        return info;
    }

    /**
     * 基础数据数据
     *
     * @return
     */
    @Override
    public Map<String, Object> getBaseData() {
        TauvDriver driver = new TauvDriver();
        Map<String, Object> map = new HashMap<>(3);
        map.put("type_list", driver.getTypeList());
        map.put("area_list", cityService.getCityList(BigInteger.valueOf(Long.valueOf("320117"))));
        map.put("level_list", driver.getLevelList());
        return map;
    }

    /**
     * 获取所有河道数据
     *
     * @return
     */
    @Override
    public Response getDriverList(Map<String, Object> map) {
        QueryWrapper queryWrapper = new QueryWrapper();
        // 区划ID
        if (map.containsKey("driverArea") && !StringUtils.isEmpty(map.get("driverArea"))) {
            queryWrapper.eq("driver_area", Integer.valueOf(map.get("driverArea").toString()));
        }
        // 河流名称
        if (map.containsKey("name") && !StringUtils.isEmpty(map.get("name"))) {
            queryWrapper.like("name", map.get("name").toString());
        }
        queryWrapper.eq("status", 1);
        queryWrapper.eq("mark", 1);

        // 起始页
        Integer pageIndex = 1;
        if (map.containsKey("page") && !StringUtils.isEmpty(map.get("page").toString())) {
            pageIndex = Integer.valueOf(map.get("page").toString());
        }
        // 每页数
        Integer pageSize = 10;
        IPage<TauvDriver> page = new Page<>(pageIndex, pageSize);
        IPage<TauvDriver> data = driverMapper.selectPage(page, queryWrapper);
        List<TauvDriver> driverList = data.getRecords();
        List<TauvDriverListVo> driverListVoList = new ArrayList<>();
        if (!driverList.isEmpty()) {
            driverList.forEach(item -> {
                TauvDriverListVo driverListVo = new TauvDriverListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, driverListVo);
                // 河流ID
                driverListVo.setDriverId(item.getId());
                // 区划描述
                String cityName = cityService.getCityName(item.getDriverArea());
                driverListVo.setDriverAreaName(cityName);
                // 河长名称
                // 河长名称
                if (item.getDriverManager() != null && item.getDriverManager() > 0) {
                    LsAdmin lsAdmin = lsAdminMapper.selectById(item.getDriverManager());
                    if (lsAdmin != null) {
                        driverListVo.setDriverManagerName(lsAdmin.getRealname());
                    }
                }
                driverListVoList.add(driverListVo);
            });
        }
        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", driverListVoList);
        return response.success(result);
    }
}
